#!/bin/sh
#   Copyright (c) 2010 digitalSTROM.org, Zurich, Switzerland
#
#   Author: Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>
#
#   This file is part of digitalSTROM Server.
#
#   netinfo.sh is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   netinfo.sh is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this file. If not, see <http://www.gnu.org/licenses/>.

# source common functions
. ./errlib.sh
. ./jsonlib.sh

HOSTNAME_FILE="/etc/hostname"
DSS_STATE_FILE="/etc/runit/dss/supervise/stat"
MT_STATE_FILE="/etc/runit/mediatomb/supervise/stat"
MT_CONFIG_FILE="/etc/mediatomb/config.xml"

restart_net_if_needed() {
    ERR=$ERR_OK
    export PATH
    OLD_SETTINGS=`./netinfo.sh console`
    MODE=`echo $OLD_SETTINGS | cut -d ":" -f 1`
    if [[ "$MODE" == "dhcp" ]]; then
# because we are not changing any settings, nothing *should* go wrong here...
# thus there are no extra fallbacks to an aliased interface
        ifdown eth0 1>&2
        ifup eth0 1>&2
        if [[ "$?" -ne "0" ]]; then
            ERR=$ERR_NETUP
        else
            sleep 1
        fi
    fi

    return $ERR_OK        
}

set_hostname() {
    echo "$1" > "$HOSTNAME_FILE"
    if [[ "$?" -ne "0" ]]; then
        json_error $ERR_FWRITE "Could not write to hostname file!"
        return $ERR_FWRITE
    fi    

    hostname -F "$HOSTNAME_FILE" 1>&2
    if [[ "$?" -ne "0" ]]; then
        json_error $ERR_HOSTNAME "Could not set hostname!"
        return $ERR_HOSTNAME
    fi

    SET_HOSTNAME=`hostname`
    if [[ "$SET_HOSTNAME" != "$1" ]]; then
        json_error $ERR_HOSTNAME "Error setting new hostname!"
        echo "dSS11" > "$HOSTNAME_FILE"
        return $ERR_HOSTNAME
    fi

    restart_net_if_needed
    RETCODE="$?"
    if [[ "$RETCODE" -ne "$ERR_OK" ]]; then
        json_error $RETCODE "Error bringing up network!"
        return $RETCODE
    fi

    return $ERR_OK
}

set_apartment_name() {
    DSS_STATE=
    if [[ -f "$DSS_STATE_FILE" ]]; then
        DSS_STATE=`cat "$DSS_STATE_FILE"`
        if [[ "$DSS_STATE" != "run" ]]; then
            echo "dSS is not running, not setting name!" 1>&2
            return 1
        fi

        JSON_RESULT=`wget -q -O - "http://127.0.0.1:8080/json/apartment/setName?newName=$1"`
        if [[ "$?" -ne "0" ]]; then
            echo "wget failed!" 1>&2
            return 1
        fi

        if [[ "$JSON_RESULT" != "{\"ok\":true}" ]]; then
            echo "dSS did not accept the name!" 1>&2
            return 1
        fi
    fi

    return 0
}

set_upnp_friendly_name() {
    if [[ -f "$MT_CONFIG_FILE" ]]; then
    
        OLD_NAME=$(cat "$MT_CONFIG_FILE" |sed -n -e 's/.*<name>\(.*\)<\/name>.*/\1/p')
        sed -i s:"<name>$OLD_NAME</name>":"<name>$1</name>":g "$MT_CONFIG_FILE"
        
        NEW_NAME=$(cat "$MT_CONFIG_FILE" |sed -n -e 's/.*<name>\(.*\)<\/name>.*/\1/p')

        if [[ "$NEW_NAME" != "$1" ]]; then
            echo "Could not set UPnP friendly name!" 1>&2
            return 1
        fi

        MT_STATE=
        if [[ -f "$MT_STATE_FILE" ]]; then
            MT_STATE=`cat "$MT_STATE_FILE"`
            if [[ "$DSS_STATE" == "run" ]]; then
                sv -w 5 down /etc/runit/mediatomb 1>&2
                sleep 1
                sv -w 15 up /etc/runit/mediatomb 1>&2
            fi
        fi

    fi

    return 0
}

echo "Content-Type: application/json"
echo ""


HOSTNAME=""

if [[ -n "$1" ]]; then 
    echo "$1" | grep "[^0-9a-zA-Z-]" > /dev/null 2>&1
    if [ "$?" -eq "0" ]; then
        json_error $ERR_HOSTNAME "Invalid hostname specified!"
        exit 1
    fi
    HOSTNAME="$1"
else
    json_error $ERR_HOSTNAME "Invalid hostname specified!"
    exit 1
fi

set_hostname "$HOSTNAME"
RETCODE="$?"
if [[ "$RETCODE" -ne "$ERR_OK" ]]; then
    exit $RETCODE
fi

set_apartment_name "$HOSTNAME"

set_upnp_friendly_name "$HOSTNAME"

json_ok
