//= require <timeinfo>
//
Ext.namespace('dSS11', 'dSS11.config');

dSS11.AccessConfigPanel = Ext.extend(Ext.Panel,
    {
        initComponent: function()
        {

            this.webPassword = new Ext.form.FieldSet(
                {
                    layout: 'form',
                    title:  _("Web interface admin password"),
                    scope:  this,
                    items:  [
                        new Ext.Panel({
                            autoEl: 'span',
                            cls: 'x-green-text',
                            autoLoad: dSS11.config.langPath + 'pwinfo.text',
                            bodyBorder: false,
                            footer: true,
                            footerCfg: { html: '<br/>' } 
                        }),
                        {
                            xtype: 'textfield',
                            width: 220,
                            name: 'currentpw',
                            ref: 'currentpw',
                            inputType: 'password',
                            allowBlank: false,
                            fieldLabel: _("Current password"),
                            maxLength: 64
                        },
                        {
                            xtype: 'textfield',
                            width: 220,
                            name: 'newpw',
                            ref: 'newpw',
                            inputType: 'password',
                            regex: new RegExp("^\\S+$"),
                            allowBlank: false,
                            fieldLabel: _("New password"),
                            maxLength: 64
                        },
                        {
                            xtype: 'textfield',
                            width: 220,
                            name: 'repeatpw',
                            ref: 'repeatpw',
                            inputType: 'password',
                            submitValue: false,
                            allowBlank: false,
                            regex: new RegExp("^\\S+$"),
                            fieldLabel: _("Repeat new password"),
                            maxLength: 64
                        },
                        {
                            xtype: 'button',
                            text: _("Apply"),
                            handler: this.setPassword.createDelegate(this)
                        }
                    ] // items
                } // field set
            );

            this.passwordForm = new Ext.FormPanel(
                {
                    padding: 6,
                    ref: 'passwordform',
                    id: 'passwordform',
                    scope: this,
                    items: [
                        this.webPassword
                    ]
                }
            );

            Ext.apply(this, {
                hideBorders: true,
                items: [
                    this.passwordForm
                ]
            });
            
            dSS11.AccessConfigPanel.superclass.initComponent.apply(this, arguments);
        }, // initcomponent
        setPassword:  function() {
            if (!this.passwordForm.getForm().isValid()) {
                Ext.Msg.alert(_("Error"), _("The password is missing or invalid!"));
                return;
            }

            if (this.webPassword.newpw.getValue() !=
                this.webPassword.repeatpw.getValue())
            {
                this.webPassword.newpw.markInvalid();
                this.webPassword.repeatpw.markInvalid();

                Ext.Msg.alert(_("Error"), _("The passwords you entered do not match!"));

                Ext.Msg.show({
                    title:_("Error"),
                    msg: _("The passwords you entered do not match!"),
                    buttons: Ext.Msg.OK,
                    scope: this,
                    fn: function() {
                        this.webPassword.newpw.reset();
                        this.webPassword.repeatpw.reset();
                        this.webPassword.newpw.markInvalid();
                        this.webPassword.repeatpw.markInvalid();
                    }
                });                
                return;
            }

            this.el.mask(_("Applying changes") + '...', 'x-mask-loading');

            this.passwordForm.getForm().submit({ 
                url: '/cgi-bin/setwebpasswrapper.sh',
                method: 'GET',
                scope: this,
                success: function(form, action) {
                    this.webPassword.currentpw.reset();
                    this.webPassword.newpw.reset();
                    this.webPassword.repeatpw.reset();
                    this.el.unmask();
                },
                failure: function(form, action) {
                    this.el.unmask();
                    switch (action.failureType) {
                        case Ext.form.Action.CONNECT_FAILURE:
                            Ext.Msg.alert(_("Error"),
                                _("Ajax communication failed"));
                        break;
                        default:
                            Ext.Msg.alert(_("Error"),
                                    _("Could not set password") + ": " +
                                    _(action.result.message));
                        break;
                    }
                }
            });
        }
    }
);

Ext.reg('dss11accessconfig', dSS11.AccessConfigPanel);

