//= require <util>

Ext.namespace('dSS11', 'dSS11.config', 'dSS11.util');

dSS11.AddOnsFeedPanel = Ext.extend(Ext.Panel,
    {
        initComponent: function()
        {
            this.feedStore = new Ext.data.JsonStore(
                {
                    url: '/cgi-bin/opkgwrapper.sh?action=addon-list',
                    root: 'data',
                    fields: 
                    [
                        'name', 'version', 'maintainer', 
                        'description', 'homepage', 'state'
                    ]
                }
            );

            this.loaded = false;
            this.refetched = false;

            var renderAddOn = function(value, p, record)
            {
                var hp = record.get('homepage');
                if (hp !== "") {
                    hp = String.format('&nbsp;&nbsp;<a href="{0}" target="_blank">{0}</a>',
                            hp);
                }

                return String.format('<div class="{0}">' + 
                        '<table border=0>' +
                        '<tr>' +
                        '<td>' +
                        '<img width="32" height="32" ' + 
                             'src="/images/default_icon.png"/>' +
                        '</td>' +
                        '<td>' +
                        '<b>{1}</b>{2}<br/>' + _("Version")+':&nbsp;{3}</div>' +
                        '<br/><div class="{4}">{5}' +
                        '</td>' +
                        '</tr>' +
                        '</table>' +
                        '</div>',
                        'x-green-text x-text-link',                 // {0}
                        record.get('name'),                         // {1}
                        hp,                                         // {2}
                        record.get('version'),                      // {3}
                        'black-text',                               // {4}
                        record.get('description')                   // {5}
                );
            };

            this.mainGrid = new Ext.grid.GridPanel(
                {
                    store: this.feedStore,
                    columns:
                    [
                        {
                            id: 'addon',
                            header: _("dS add-ons"),
                            dataIndex: 'name',
                            sortable: true,
                            renderer: renderAddOn
                        }
                    ],
                    trackMouseOver: false,
                    viewConfig: { forceFit: true },
                    sm: new Ext.grid.RowSelectionModel({singleSelect:true}),
                    bodyBorder: false
                }
            );

            this.refetchTip = new Ext.ToolTip(
                {
                    target: 'refetchButton',
                    title: _("Update list of packages"),
                    html: '<br/>' +
                         _("Make sure to click this button to update the " + 
                           "list of available packages."),
                    closable: true,
                    draggable: true,
                    anchor: 'top'
                }
            );

            Ext.apply(this,
                {
                    layout: 'fit',
                    tbar: [
                        {
                            xtype: 'button',
                            text: _("Refresh view"),
                            iconCls: 'icon-refresh-view',
                            scope: this,
                            handler: this.refreshList.createDelegate(this)
                        },
                        {
                            xtype: 'button',
                            text: _("Refetch list from server"),
                            id: 'refetchButton',
                            iconCls: 'icon-refresh-feed',
                            tooltip: _("Download latest package information " +
                                       "list from the server"),
                            scope: this,
                            handler: this.handleRefetch.createDelegate(this)
                        },
                        '->',
                        {
                            xtype: 'button',
                            text: _("Install selected"),
                            tooltip: _("Download and install selected add-on"),
                            iconCls: 'icon-package-add',
                            id: 'installButton',
                            scope: this,
                            handler: this.installSelected.createDelegate(this),
                            disabled: true
                        }
                    ],
                    
                    items: [ this.mainGrid ]
                }
            );

            dSS11.AddOnsFeedPanel.superclass.initComponent.apply(this, arguments);

            this.on('activate',
                function() {
                    if (this.loaded === false) {
                        this.feedStore.load();
                        this.loaded = true;
                    }
                },
                this);

            this.feedStore.on('load',
                function() 
                {
                    if ((this.feedStore.getTotalCount() === 0) && 
                        (this.refetched === false)) {
                        this.refetchTip.show();
                    }
                },
                this);

            this.mainGrid.getSelectionModel().on('selectionchange',
                function(sel)
                {
                    if (sel.hasSelection()) {
                        this.getTopToolbar().get('installButton').enable();
                    } else {
                        this.getTopToolbar().get('installButton').disable();
                    }
                },
                this);

        },
        refreshList: function() 
        {
            this.feedStore.load();
        },
        refetchDone: function()
        {
            this.refetched = true;
            this.refetchTip.hide();
            this.refetchTip.disable();
            this.refreshList();
        },
        handleRefetch: function() 
        {
            var fn = dSS11.util.opkg.createDelegate(this, 
                           [ 'addon-update', 
                              this.findParentByType('viewport').el,
                              this.refetchDone.createDelegate(this) ]);
            fn();
        },
        installSelected: function()
        {
            var sel = this.mainGrid.getSelectionModel();
            if (!sel.hasSelection()) {
                Ext.MessageBox.alert(_("No add-on selected"),
                      _("Please select an add-on that should be installed!"));
                return;
            }

            record = sel.getSelected();
            var name = record.get('name');

            var fn = dSS11.util.opkg.createDelegate(this,
                    [ 'addon-install',
                       this.findParentByType('viewport').el,
                       this.doneInstall.createDelegate(this, [ name ]),
                       name ]);
            fn();
        },
        doneInstall: function(name) {
            Ext.Msg.show({
                title: _("Info"),
                msg: _("Add-on") + " <b>" + name + "</b> " +
                     _("was successfully installed on your system"),
                icon: Ext.MessageBox.INFO,
                buttons: Ext.Msg.OK
            });
            this.refreshList();
            this.findParentByType('dss11addons').overview.refreshOverview();
        }
        
    }
);

Ext.reg('dss11addonsfeed', dSS11.AddOnsFeedPanel);
