Ext.namespace('dSS11', 'dSS11.config');

dSS11.AddOnsOverviewPanel = Ext.extend(Ext.Panel,
    {
        initComponent: function()
        {
            this.installedStore = new Ext.data.JsonStore(
                {
                    url: '/cgi-bin/opkgwrapper.sh?action=addon-list-installed',
                    root: 'data',
                    fields: 
                    [
                        'name', 'version', 'maintainer', 
                        'description', 'homepage'
                    ]
                }
            );

            this.loaded = false;

            this.mainView = new Ext.DataView(
                {
                    store: this.installedStore,
                    id: 'add-on-installed-view',
                    scope: this,
                    overClass: 'x-view-over',
                    loadingText: _("Loading list of installed add-ons") + "...",
                    emptyText: _("You do not have any add-ons installed"),
                    deferEmptyText: true,
                    cls: 'x-margined-green-text',
                    tpl: new Ext.XTemplate(
                        '<tpl for=".">',
                        '<div qtip="<b>{name}<br/>Version: ' + 
                        '</b>{version}<br/><br/>' + 
                        '<i>{description}</i>" style="float: left">',
                        '<div class="addon-cell-item-wrap" id="{name}">',
                        '<div class="addon-cell-item">',
                        '<img src="/add-ons/{name}/default_icon.png"/>',
                        '</div>',
                        '<div class="x-green-text">',
                        '<span>{name}</span>',
                        '</div>',
                        '</div>',
                        '</div>',
                        '</tpl>',
                        '<div class="x-clear"></div>'

                    ),
                    itemSelector: 'div.addon-cell-item-wrap',
                    singleSelect: true,
                    autoScroll: true,
                    listeners: {
                        dblclick: this.forwardToAddon.createDelegate(this)
                    }
                    //plugins: [ new Ext.DataView.DragSelector() ]
                }
            );

            Ext.apply(this,
                {
                    layout: 'fit',
                    tbar: [
                        {
                            xtype: 'button',
                            text: _("Refresh view"),
                            iconCls: 'icon-refresh-view',
                            scope: this,
                            handler: this.refreshOverview.createDelegate(this)
                        },
                        {
                            xtype: 'button',
                            text: _("Open selected"),
                            iconCls: 'icon-package-open',
                            id: 'openButton',
                            tooltip: _("Hint: double click on the add-on icon to open the configuration interface"),
                            handler: this.forwardToSelected.createDelegate(this),
                            disabled: true,
                            scope: this
                        },
                        '->',
                        {
                            text: _("Uninstall selected"),
                            scope: this,
                            tooltip: _("Remove selected add-ons from your system"),
                            iconCls: 'icon-package-delete',
                            handler: this.removeAddons.createDelegate(this),
                            disabled: true,
                            id: 'uninstallButton'
                        }
                    ],
                    items: [ this.mainView ]
                }
            );

            dSS11.AddOnsOverviewPanel.superclass.initComponent.apply(this, 
                                                                     arguments);
            this.on('activate',
                function() {
                    if (this.loaded === false) {
                        this.installedStore.load();
                        this.loaded = true;
                    }
                },
                this);

            this.mainView.on('selectionchange',
                function(dv, selections)
                {
                    if (selections.length > 0) {
                        this.getTopToolbar().get('openButton').enable();
                        this.getTopToolbar().get('uninstallButton').enable();
                    } else {
                        this.getTopToolbar().get('openButton').disable();
                        this.getTopToolbar().get('uninstallButton').disable();
                    }
                },
                this);

        }, // initcomponent
        forwardToAddon: function(dv, index, node, ev) {
            window.open('/add-ons/' + dv.getRecord(node).get('name') + '/');
        },
        forwardToSelected: function()
        {
            if (this.mainView.getSelectionCount() === 0) {
                Ext.MessageBox.alert(_("No add-on selected"),
                      _("Please select an add-on to open! You can also double click the add-on icon to open the configuration interface"));
                return;
            }
            var selection = this.mainView.getSelectedRecords();
            var name = selection[0].get('name');
            window.open('/add-ons/' + name + '/');
        },
        removeAddons: function() {
            if (this.mainView.getSelectionCount() === 0) {
                Ext.MessageBox.alert(_("No add-on selected"),
                      _("Please select an add-on that should be uninstalled!"));
                return;
            }
            Ext.Msg.show({
                title: _("Uninstall add-ons"),
                msg: _("Are you sure that you want to uninstall the " + 
                       "selected add-on from your system?"),
                buttons: Ext.Msg.YESNO,
                scope: this,
                fn: function(button, text) {
                    if (button === 'yes') {
                        var selection = this.mainView.getSelectedRecords();
                        var name = selection[0].get('name');

                        var fn = dSS11.util.opkg.createDelegate(this,
                            [ 'addon-remove',
                               this.findParentByType('viewport').el,
                               this.doneRemove.createDelegate(this, [ name ]),
                               name ]);
                        fn();
                    }
                }
            });
        },
        refreshOverview: function() {
            this.installedStore.load();
        },
        doneRemove: function(name) {
            Ext.Msg.show({
                title: _("Info"),
                msg: _("Add-on") + " <b>" + name + "</b> " + 
                     _("was removed from your system"),
                icon: Ext.MessageBox.INFO,
                buttons: Ext.Msg.OK
            });
            this.refreshOverview();
        }
    }
);

Ext.reg('dss11addonsoverview', dSS11.AddOnsOverviewPanel);
