Ext.namespace('dSS11', 'dSS11.config');

dSS11.AddOnsUpdatePanel = Ext.extend(Ext.Panel,
    {
        initComponent: function()
        {
            this.updateStore = new Ext.data.JsonStore(
                {
                    url: '/cgi-bin/opkgwrapper.sh?action=addon-list-upgradable',
                    root: 'data',
                    fields: 
                    [
                        'name', 'version', 'maintainer', 
                        'description', 'homepage', 'state'
                    ]
                }
            );

            this.loaded = false;
            this.fromRefetch = false;

            var renderAddOn = function(value, p, record)
            {
                var hp = record.get('homepage');
                if (hp !== "") {
                    hp = String.format('&nbsp;&nbsp;<a href="{0}" target="_blank">{0}</a>',
                            hp);
                }

                return String.format('<div class="{0}">' + 
                        '<table border=0>' +
                        '<tr>' +
                        '<td>' +
                        '<img width="32" height="32" ' + 
                             'src="/images/default_icon.png"/>' +
                        '</td>' +
                        '<td>' +
                        '<b>{1}</b>{2}<br/>'+_("Version")+':&nbsp;{3}</div>' +
                        '<br/><div class="{4}">{5}' +
                        '</td>' +
                        '</tr>' +
                        '</table>' +
                        '</div>',
                        'x-green-text x-text-link',                 // {0}
                        record.get('name'),                         // {1}
                        hp,                                         // {2}
                        record.get('version'),                      // {3}
                        'black-text',                               // {4}
                        record.get('description')                   // {5}
                );
            };

            this.mainGrid = new Ext.grid.GridPanel(
                {
                    store: this.updateStore,
                    columns:
                    [
                        {
                            id: 'addon',
                            header: _("dS add-ons"),
                            dataIndex: 'name',
                            sortable: true,
                            renderer: renderAddOn
                        }
                    ],
                    trackMouseOver: false,
                    viewConfig: { forceFit: true },
                    sm: new Ext.grid.RowSelectionModel({singleSelect:true}),
                    bodyBorder: false
                }
            );

            this.noUpdatesTooltip =  new Ext.ToolTip(
                {
                    target: 'updateButton',
                    html: '<br/>' + 
                          _("Your add-ons are up to date"),
                    title: _("No updates found"),
                    closable: true,
                    draggable: true,
                    anchor: 'right'
                }
            );

            this.gotUpdatesTooltip = new Ext.ToolTip(
                {
                    target: 'updateButton',
                    html: '<br/>' + 
                          _("Add-on updates are available for your system, click the update button to install them"),
                    title: _("Updates found"),
                    closable: true,
                    draggable: true,
                    anchor: 'right'
                }
            );

            Ext.apply(this,
                {
                    layout: 'fit',
                    tbar: [
                        {
                            xtype: 'button',
                            text: _("Refresh view"),
                            iconCls: 'icon-refresh-view',
                            scope: this,
                            handler: this.refreshList.createDelegate(this)
                        },
                        {
                            xtype: 'button',
                            text: _("Check for new updates"),
                            iconCls: 'icon-refresh-feed',
                            scope: this,
                            tooltip: _("Contact the server and check availability of new updates"),
                            handler: this.handleRefetch.createDelegate(this)
                        },
                        '->',
                        {
                            xtype: 'button',
                            text: _("Update all"),
                            ref: 'updateButton',
                            id: 'updateButton',
                            iconCls: 'icon-package-update',
                            disabled: true,
                            scope: this,
                            handler: this.updateAddons.createDelegate(this)
                        },
                        {
                            xtype: 'button',
                            text: _("Update selected"),
                            ref: 'updateSelButton',
                            id: 'updateSelButton',
                            iconCls: 'icon-package-update',
                            disabled: true,
                            scope: this,
                            handler: this.updateSelectedAddon.createDelegate(this)
                        }
                    ],
                    items: [ this.mainGrid ]
                }
            );

            dSS11.AddOnsUpdatePanel.superclass.initComponent.apply(this, arguments);

            this.on('activate',
                function() 
                {
                    if (this.loaded === false) {
                        this.updateStore.load();
                        this.loaded = true;
                    }
                },
                this);

            this.on('hide',
                function()
                {
                    this.noUpdatesTooltip.hide();
                    this.gotUpdatesTooltip.hide();
                },
                this);

            this.updateStore.on('load',
                function() 
                {
                    var button = this.getTopToolbar().get('updateButton');
                    var selButton = this.getTopToolbar().get('updateSelButton');
                    var count = this.updateStore.getTotalCount();

                    if (count > 0) {
                        this.noUpdatesTooltip.hide();
                        this.noUpdatesTooltip.disable();
                        this.gotUpdatesTooltip.enable();
                        button.enable();
                        this.gotUpdatesTooltip.setTitle(
                                count + " " + 
                                dSS11.config.gt.ngettext("Update", 
                                                    "n% Updates", count) +
                                        " " + _("found"));

                        this.gotUpdatesTooltip.show();
                    } else {
                        button.disable();
                        selButton.disable();
                        this.gotUpdatesTooltip.hide();
                        this.gotUpdatesTooltip.disable();
                        this.noUpdatesTooltip.enable();
                        if (this.fromRefetch === true) {
                            this.noUpdatesTooltip.show();
                        }
                    }

                    if (this.fromRefetch === true) {
                        this.findParentByType('dss11addons').feeds.refreshList();
                        this.fromRefetch = false;
                    }
                },
                this);

            this.mainGrid.getSelectionModel().on('selectionchange',
                function(sel)
                {
                    if (sel.hasSelection()) {
                        this.getTopToolbar().get('updateSelButton').enable();
                    } else {
                        this.getTopToolbar().get('updateSelButton').disable();
                    }
                },
                this);
        },
        refreshList: function() 
        {
            this.updateStore.load();
        },
        updateDone: function(name)
        {
            var msg = _("Add-ons successfully updated");
            if (name !== undefined) {
                msg = _("Add-on") + ' <b>' + name + '</b> ' + 
                      _("successfully updated");
            }
            Ext.Msg.show({
                title: _("Info"),
                msg: msg,
                icon: Ext.MessageBox.INFO,
                buttons: Ext.Msg.OK
            });
            this.refreshList();
            this.findParentByType('dss11addons').overview.refreshOverview();
            this.findParentByType('dss11addons').feeds.refreshList();
        },
        refetchDone: function() 
        {
            this.fromRefetch = true;
            this.refreshList();
        },
        handleRefetch: function()
        {
            var fn = dSS11.util.opkg.createDelegate(this,
                           [ 'addon-update',
                              this.findParentByType('viewport').el,
                              this.refetchDone.createDelegate(this) ]);
            fn();
        },
        updateAddons: function() {
            var fn = dSS11.util.opkg.createDelegate(this,
                           [ 'addon-upgrade',
                              this.findParentByType('viewport').el,
                              this.updateDone.createDelegate(this) ]);
            fn();
        },
        updateSelectedAddon: function() {
            var sel = this.mainGrid.getSelectionModel();
            if (!sel.hasSelection()) {
                Ext.MessageBox.alert(_("No add-on selected"),
                        _("Please select an add-on that should be updated!"));
                return;
            }

            record = sel.getSelected();
            var name = record.get('name');

            var fn = dSS11.util.opkg.createDelegate(this,
                           [ 'addon-upgrade',
                              this.findParentByType('viewport').el,
                              this.updateDone.createDelegate(this, [ name ]),
                              name ]);
            fn();
        }

    }
);

Ext.reg('dss11addonsupdate', dSS11.AddOnsUpdatePanel);
