//= require <netconfpanel>
//= require <sysconfpanel>
//= require <accesspanel>
//= require <systemupdate>
//= require <addons>

Ext.namespace('dSS11');

dSS11.NavPanel = Ext.extend(Ext.Panel, 
    {
        initComponent: function() 
        {

            var tabRecord = Ext.data.Record.create([ 
                    { 
                        name: "title", 
                        id: "id",
                        auto: "auto"
                    } 
                ]);

            var tabStore = new Ext.data.Store({}, tabRecord);

            var cpItems =
                [
                    {
                        title: _("Overview"),
                        id: 'overview',
                        xtype: 'panel',
                        hideBorders: true,
                        ref: 'overview',
                        autoLoad: 'cgi-bin/prodinfo.hs',
                        auto: true
                    },
                    {
                        title: _("dS add-ons"),
                        id: 'addons',
                        xtype: 'panel',
                        hideBorders: true,
                        ref: 'addons',
                        xtype: 'dss11addons',
                        auto: false
                    },
                    {                    
                        title: _("Logged in users"),
                        id: 'loggedinusers',
                        hideBorders: true,
                        xtype: 'panel',
                        ref: 'loggedinusers',
                        autoLoad: 'cgi-bin/who.hs',
                        auto: true
                    },
                    {
                        title: _("Network settings"),
                        id: 'netconfig',
                        ref: 'netconfig',
                        hideBorders: true,
                        xtype: 'dss11netconfig',
                        auto: false
                    },
                    {
                        title: _("System settings"),
                        id: 'sysconfig',
                        ref: 'sysconfig',
                        hideBorders: true,
                        xtype: 'dss11sysconfig',
                        auto: false
                    },
                    {
                        title: _("Access settings"),
                        id: 'accessconfig',
                        ref: 'accessconfig',
                        hideBorders: true,
                        xtype: 'dss11accessconfig',
                        auto: false
                    },
                    {
                        title: _("System update"),
                        id: 'sysupdate',
                        ref: 'sysupdate',
                        hideBorders: true,
                        xtype: 'dss11systemupdate',
                        auto: false
                    }

                ];

            this.defaultSelection = 0;

            var params = Ext.urlDecode(window.location.search.substring(1));

            var param = parseInt(params.select, 10);
            if ((param >= 0) && (param < cpItems.length)) {
                this.defaultSelection = param;
            }


            var contentPanel = 
            {
                ref: 'contentPanel',
                region: 'center',
                layout: 'card',
                hideBorders: true,
                autoScroll: true,
                split: true,
                items: cpItems
            };

            this.listView = new Ext.list.ListView(
                {
                    ref: 'listView',
                    store: tabStore,
                    singleSelect: true,
                    region: 'west',
                    hideHeaders: true,
                    columns: [{ header: 'name', dataIndex: 'title' }]
                }
            );

            var listHolder = new Ext.Panel(
                {
                    title: _("Menu"),
                    region: 'west',
                    layout: 'fit',
                    width: 190,
                    hideBorders: true,
                    split: true,
                    items: [ this.listView ] 
                }
            );

            Ext.apply(this, 
                {
                    layout: 'border',
                    items: [ listHolder, contentPanel ]
                }
            );

            dSS11.NavPanel.superclass.initComponent.apply(this, arguments);

            this.on(
                'beforerender',
                function(component) 
                {
                    if (this.initializedTabs) 
                    {
                        return;
                    }
        
                    // for each item in contentPanel
                    this.contentPanel.items.each(
                        function(item) 
                        {
                            var tab = new tabRecord(
                                {
                                    title: item.title,
                                    id: item.id,
                                    auto: item.auto
                                }
                            );
                            tabStore.add([tab]);
                        }, 
                        this
                    );

                    this.listView.on(
                        'afterrender',
                        function() 
                        {
                            this.listView.select(this.defaultSelection); 
                        },
                        this
                    );

                    this.listView.on(
                        'selectionchange',
                        function(listView, selections) 
                        {
                            this.contentPanel.layout.setActiveItem(selections[0].viewIndex);
                        },
                        this
                    );

                    this.listView.on(
                        'click',
                        function(listView, index)
                        {
                            if (listView.getRecord(listView.getNode(index)).get('auto') === true) {
                                Ext.getCmp(listView.getRecord(listView.getNode(index)).get('id')).doAutoLoad(); 
                            }
                        }
                    ); 
                    this.initializedTabs = true;
                },
                this
            ); // this.on

        } // initcomponent
    } // extend
);

Ext.reg('dss11navpanel', dSS11.NavPanel);
