Ext.namespace('dSS11', 'dSS11.config');

dSS11.NetConfigPanel = Ext.extend(Ext.Panel, 
    {
        initComponent: function() 
        {
            this.url = '/cgi-bin/netinfo.sh';

            this.backedUp = false;

            this.formData = {};

            this.ipset = new Ext.form.FieldSet(
                {
                    layout: 'form',
                    ref:    'ipset',
                    scope: this,
                    items: [
                            new Ext.form.TextField({
                                 width: 140,
                                 name: 'ip',
                                 allowBlank: false,
                                 fieldLabel: _("IP Address"),
                                 validator: this.validateIP
                            }),
                            new Ext.form.TextField({
                                 width: 140,
                                 name: 'netmask',
                                 allowBlank: false,
                                 fieldLabel: _("Netmask"),
                                 validator: this.validateIP
                            }),
                            new Ext.form.TextField({
                                 width: 140,
                                 name: 'gateway',
                                 allowBlank: true,
                                 fieldLabel: _("Gateway"),
                                 validator: this.validateIPBlankOK.createDelegate(this)
                            }),
                            new Ext.form.TextField({
                                 width: 140,
                                 name: 'dns1',
                                 allowBlank: true,
                                 fieldLabel: _("DNS") + "1",
                                 validator: this.validateIPBlankOK.createDelegate(this)
                            }),
                            new Ext.form.TextField({
                                 width: 140,
                                 name: 'dns2',
                                 allowBlank: true,
                                 fieldLabel: _("DNS") + "2",
                                 validator: this.validateIPBlankOK.createDelegate(this)
                            }),
                            new Ext.form.DisplayField({
                                width: 140,
                                ref: 'mac',
                                name: 'mac',
                                fieldLabel: _("MAC")
                            })
                        ] // ipset items

                }
            );

            this.ipform = new Ext.FormPanel({
                        padding: 6,
                        ref: 'ipform',
                        id: 'ipform',
                        url: '/cgi-bin/netcfgwrapper.sh',
                        items: [
                        new Ext.Panel({
                            autoEl: 'span',
                            cls: 'x-green-text',
                            autoLoad: dSS11.config.langPath + 'netconf.text',
                            bodyBorder: false
                        }),
                            {
                                 xtype: 'panel',
                                 height: 10,
                                 bodyBorder: false
                            },

                            new Ext.CycleButton({
                                width: 400,
                                showText: true,
                                id: 'mode',
                                ref: 'mode',
                                items: [
                                            {
                                                xtype: 'menucheckitem',
                                                text: 
                                                _("Automatic IP address " +
                                                    "configuration " +
                                                    "using DHCP"),
                                                id: 'ipdhcp',
                                                ref: 'ipdhcp'
                                            },
                                            {
                                                xtype: 'menucheckitem',
                                                text: _("Static IP address " +
                                                        "configuration"),
                                                id: 'ipstatic',
                                                ref: 'ipstatic'
                                            }
                                ],
                                changeHandler: this.netCycle,
                                scope: this
                            }),
                            {
                                 xtype: 'panel',
                                 height: 10,
                                 bodyBorder: false
                            },
                            {
                                xtype: 'panel',
                                ref: 'message',
                                id: 'message',
                                height: 20,
                                cls: 'x-green-text',
                                autoEl: 'span',
                                bodyBorder: false
                            },
                            this.ipset,
                            {
                                xtype: 'panel',
                                layout: 'border',
                                bodyBorder: false,
                                height: 35,
                                items: [
                                    {
                                        xtype: 'panel',
                                        region: 'center',
                                        bodyBorder: false,
                                        width: 150,
                                        buttonAlign: 'left',
                                        minButtonWidth: 80,
                                        buttons: [
                                            {
                                                xtype: 'button',
                                                text:  _("Refresh"),
                                                handler: this.reloadData.createDelegate(this)
                                            }
                                        ]
                                    },
                                    {
                                        xtype: 'panel',
                                        region: 'east',
                                        bodyBorder: false,
                                        buttonAlign: 'right',
                                        minButtonWidth: 80,
                                        buttons: [
                                        {
                                            xtype: 'button',
                                            text:  _("Apply"),
                                            handler: this.applyChanges.createDelegate(this)
                                        },
                                        {
                                            xtype: 'button',
                                            text:  _("Cancel"),
                                            handler: this.cancelChanges,
                                            scope: this
                                        },
                                        {
                                            xtype: 'button',
                                            text:  _("Help"),
                                            handler: this.help
                                        }
                                    ]
                                }
                              ]
                            }
                        ] // form items
                    });

            this.msgready = false;

            Ext.apply(this, {
                hideBorders: true,
                items: [
                    this.ipform
                ]
            });

            dSS11.NetConfigPanel.superclass.initComponent.apply(this, arguments);
            this.on('activate',
                    function() {
                        this.msgready = true;
                        this.reloadData(1000, this);
                        // try to backup the currently working configuration, 
                        // but ignore any errors if this fails, at this stage 
                        // it's not critical
                        // basically we need it only once, the very first 
                        // time, because if noone did any network changes 
                        // yet - we will not have a valid backup
                    },
                    this);

            this.on('beforeshow',
                    function() {
                        if (this.backedUp === false) {
                            Ext.Ajax.request({
                                url: '/cgi-bin/netcfgwrapper.sh',
                                disableCaching: true,
                                method: 'GET',
                                scope: this,
                                params: { mode: 'backup' },
                                success: function() {
                                    this.backedUp = true;
                                },
                                failure: function() {}
                            });
                        }
                    },
                    this);

        }, // initcomponent

    reloadData: function() {
        this.el.mask(_("Loading current network configuration") + "...", 
                      'x-mask-loading');
        this.ipform.load({
                url: this.url,
                method: "GET",
                scope: this,
                disableCaching: true,
                success: function(form, action) {
                    this.el.unmask();
                    try {
                        var jsonData = Ext.util.JSON.decode(
                                                action.response.responseText);

                        if (jsonData.success !== true) {
                            Ext.Msg.alert(_("Error"), jsonData.message);
                            return;
                        }

                        this.formData = jsonData.data;

                        if (jsonData.data.mode === 'static') {
                            this.ipform.mode.setActiveItem(
                                              this.ipform.mode.menu.ipstatic);
                        } else {
                            this.ipform.mode.setActiveItem(
                                              this.ipform.mode.menu.ipdhcp);
                        }
                    } catch (error) {
                        Ext.Msg.alert(_("Error"), 
                            _("Could not load network settings") + 
                                ": "  + error.message);
                    }
                },
                failure: function(form, action) {
                    this.el.unmask();
                    switch (action.failureType) {
                        case Ext.form.Action.CONNECT_FAILURE:
                            Ext.Msg.alert(_("Error"), 
                                _("Ajax communication failed"));
                        break;
                        default:
                            Ext.Msg.alert(_("Error"),
                                    _("Could not load network settings"));
                        break;
                    }
                }
        });
    },

    validateIP: function(ip) {
        if (ip === undefined) {
            return false;
        }

        ip = ip.replace(/\s/g, "");
        var re = /^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$/;
        if (re.test(ip)) {
            var parts = ip.split(".");
            if (parts.lengh < 4) {
                return false;
            }

            var n1 = parseInt(parseFloat(parts[0]), 10);
            var n2 = parseInt(parseFloat(parts[1]), 10);
            var n3 = parseInt(parseFloat(parts[2]), 10);
            var n4 = parseInt(parseFloat(parts[3]), 10);

            if (n1 === n2 === n3 === n4 === 255) {
                return false;
            }

            if (n1 === n2 === n3 === n4 === 0) {
                return false;
            }

            if (n1 === 0) {
                return false;
            }

            if (n4 === 255) {
                return false;
            }

            for (var i = 0; i < parts.length; i++) {
                var num = parseInt(parseFloat(parts[i]), 10);
                if ((num > 255) || (num < 0)) {
                    return false;
                }
            }
            return true;
        } else {
            return false;
        }                    
    },

    validateIPBlankOK: function(ip) {
        if ((ip === undefined) || (ip.length === 0)) {
            return true;
        }

        return this.validateIP(ip);
    },

    netCycle: function(btn, item) {
        if (item.id === 'ipdhcp') {
            if (this.msgready === true) {
                this.ipform.message.update(_("Your network settings will be configured automatically via DHCP."));
            }
            for (var i = 0; i < this.ipset.items.items.length; i++) {
                if (this.ipset.items.items[i].name !== 'mac') {
                    this.ipset.items.items[i].disable();
                }
            }

        } else if (item.id === 'ipstatic') {
            if (this.msgready === true) {
                this.ipform.message.update(_("Please configure your network settings manually:"));
            }
            for (var j = 0; j < this.ipset.items.items.length; j++) {
                this.ipset.items.items[j].enable();
            }

         }
    },
    cancelChanges: function() {
        this.ipform.getForm().setValues(this.formData);

        if ((this.formData.mode === 'static') && (this.ipform.mode.getActiveItem() != this.ipform.mode.menu.ipstatic)) {
            this.ipform.mode.setActiveItem(this.ipform.mode.menu.ipstatic);
        } else if ((this.formData.mode === 'dhcp') && (this.ipform.mode.getActiveItem() != this.ipform.mode.menu.ipdhcp)) {
            this.ipform.mode.setActiveItem(this.ipform.mode.menu.ipdhcp);
        }
    },
    restoreSettings: function() {
        Ext.Ajax.request({
            url: '/cgi-bin/netcfgwrapper.sh',
            disableCaching: true,
            method: 'GET',
            params: { mode: 'restore' },
            scope: this,
            success: function() { this.reloadData(); },
            failure: function() {
                Ext.Msg.alert(_("Error"), _("Could not restore last working configuration!"));
                this.reloadData();
            }
        });
    },
    applyChanges: function() {
        if (!this.ipform.getForm().isValid()) {
            Ext.Msg.alert(_("Error"), _("Please correct the errors in the red marked fields."));
            return;
        }
        this.el.mask(_("Applying changes") + '...', 
                      'x-mask-loading');

        var lastSettings = this.formData;

        this.ipform.getForm().submit({
            url: this.ipform.url, method: 'GET', 
            scope: this,
            params: { 'mode' : this.ipform.mode.getActiveItem().id },
            success: function(form, action) {
                this.el.unmask();
                try {
                    var jsonData = Ext.util.JSON.decode(
                                            action.response.responseText);

                    if (jsonData.code === dSS11.config.codes.ERR_REDIRECT) {
                        if (jsonData.ip.length > 0) {
                            this.el.mask(_("Redirecting to new location at") + 
                                " http://" + jsonData.ip + "/",
                                                          'x-mask-loading');

                            window.location = 'http://' + jsonData.ip + 
                                              '/?select=3';
                        }
                    } else {
                        this.reloadData();
                    }
                } catch (error) {
                        Ext.Msg.alert(_("Error"), 
                            _("Could not load network settings") + 
                                ": "  + error.message);
                }
            },
            failure: function(form, action) {
                this.el.unmask();
                if (action.failureType === Ext.form.Action.CONNECT_FAILURE) {
                    Ext.Msg.alert(_("Error"), _("Ajax communication failed"));
                    return;
                }

                try {
                    var jsonData = Ext.util.JSON.decode(
                                            action.response.responseText);
                    if (jsonData.code == dSS11.config.codes.ERR_NETUP) {
                        Ext.Msg.show({
                            title: _("Error"),
                            msg: _("There was an error applying your " + 
                                "new network settings! Do you want to " +
                                "revert to your previous working " + 
                                "configuration?"),
                                buttons: Ext.Msg.YESNO,
                                scope: this,
                                fn: function(button, text) {
                                    if (button == 'yes') {
                                        this.formData = lastSettings;
                                        this.cancelChanges();
                                        this.restoreSettings();
                                    } else {
                                        this.reloadData();
                                    }
                                },
                                animEl: 'elId',
                                icon: Ext.MessageBox.QUESTION
                            });
                        return;
                    }

                    Ext.Msg.alert(_("Error"), jsonData.message);
                    return;

                } catch (error) {
                        Ext.Msg.alert(_("Error"), 
                            _("Could not load network settings") + 
                                ": "  + error.message);
                }
            }
        });
    },
    help: function() {
    }
});

Ext.reg('dss11netconfig', dSS11.NetConfigPanel);
