//= require <timeinfo>
//
Ext.namespace('dSS11', 'dSS11.config');

dSS11.SysConfigPanel = Ext.extend(Ext.Panel,
    {
        initComponent: function()
        {

            this.sysName = new Ext.form.FieldSet(
                {
                    layout: 'form',
                    title:  _("Installation name"),
                    scope:  this,
                    items:  [
                        new Ext.Panel({
                            autoEl: 'span',
                            cls: 'x-green-text',
                            autoLoad: dSS11.config.langPath + 'nameconf.text',
                            bodyBorder: false,
                            footer: true,
                            footerCfg: { html: '<br/>' } 
                        }),
                        {
                            layout: 'column',
                            bodyBorder: false,
                            items: [
                                {
                                    layout: 'form',
                                    bodyBorder: false,
                                    width: 335,
                                    items: [
                                        {
                                            xtype: 'textfield',
                                            width: 220,
                                            name: 'hostname',
                                            allowBlank: false,
                                            fieldLabel: _("Name"),
                                            maxLength: 64,
                                            validator: this.validateHostName
                                        }
                                    ]
                                }, 
                                {
                                    bodyBorder: false,
                                    items: [
                                        {
                                            xtype: 'button',
                                            text: _("Apply"),
                                            handler: this.setSystemName.createDelegate(this)
                                        }
                                    ]
                                }
                            ] // border panel items
                        } // column panel
                    ] // name field set items 
                } // name field set
            );

            this.nameForm = new Ext.FormPanel(
                {
                    padding: 6,
                    ref: 'nameform',
                    id: 'nameform',
                    scope: this,
                    items: [
                        this.sysName
                    ]
                }
            );

            this.zoneStore = new Ext.data.JsonStore({
                //autoDestroy: true,
                url: '/cgi-bin/zonelist.sh',
                root: 'data.zonelist',
                fields: [ 'zone' ]
            });
 
            this.timePanel = new Ext.Panel(
                {
                    layout: 'table',
                    width: 400,
                    layoutConfig: {
                        columns: 1,
                        tableAttrs: {
                            width: '100%'
                        }
                    },
                    scope: this,
                    bodyBorder: false,
                    items: [
                        {
                            xtype: 'checkbox',
                            name: 'ntp',
                            ctCls: 'x-green-text black-text',
                            hideBorders: true,
                            boxLabel: _("Synchronize date and time over the network"),
                            checked: true,
                            listeners: {
                                check: this.NTPChecked.createDelegate(this)
                            }
                        },
                        {
                            xtype: 'panel',
                            bodyBorder: false,
                            layout: 'fit',
                            ref: 'ntpPanelSwitch',
                            items: [
                                {
                                    xtype: 'panel',
                                    layout: 'form',
                                    scope: this,
                                    bodyBorder: false,
                                    ref: 'ntpPanel',
                                    items: [
                                        {
                                            xtype: 'textfield',
                                            fieldLabel: _("NTP server"),
                                            allowBlank: false,
                                            name: 'ntpserver',
                                            width: 220
                                        }
                                    ]
                                }, // ntp panel
                                {
                                    xtype: 'panel',
                                    ref: 'datePanel',
                                    scope: this,
                                    layout: 'form',
                                    bodyBorder: false,
                                    hidden: true,
                                    items: [
                                        {
                                            xtype: 'datefield',
                                            fieldLabel: _("Date"),
                                            ref: 'date',
                                            name: 'setdate',
                                            format: 'd-m-Y',
                                            width: 220 
                                        },
                                        {
                                            xtype: 'timefield',
                                            fieldLabel: _("Time"),
                                            ref: 'time',
                                            format: 'H:i:s',
                                            name: 'settime',
                                            width: 220
                                        }
                                    ]
                                } // date panel
                            ]
                        } // panel with all ntp and date/time settings
                    ] // fieldset items
                }
            );

            this.timeSet = new Ext.form.FieldSet(
                {
                    title: _("Date and time settings"),
                    scope: this,
                    layout: 'table',
                    layoutConfig: {
                        columns: 1,
                        tableAttrs: {
                            width: '100%'
                        }
                    },
                    
                    items: [
                        {
                            xtype: 'panel',
                            ref: 'current',
                            id: 'currentTime',
                            cls: 'x-green-text',
                            autoEl: 'span',
                            height: 20,
                            bodyBorder: false
                        },
                        {
                            xtype: 'panel',
                            height: 10,
                            bodyBorder: false
                        },
                        {
                            xtype: 'panel',
                            scope: this,
                            layout: 'form',
                            bodyBorder: false,
                            items: [
                                {
                                    xtype: 'combo',
                                    fieldLabel: _("Time zone"),
                                    lazyInit: false,
                                    mode: 'local',
                                    name: 'timezone',
                                    forceSelection: true,
                                    store: this.zoneStore,
                                    emptyText: _("Select a time zone")+
                                              "...",
                                    triggerAction: 'all',
                                    typeAhead: true,
                                    displayField: 'zone',
                                    valueField: 'zone',
                                    width: 220
                                }
                            ]
                        }, // time zone panel

                        this.timePanel,
                        {
                            bodyBorder: false,
                            xtype: 'panel',
                            width: 335,
                            buttons: [
                                {
                                    xtype: 'button',
                                    text: _("Apply"),
                                    handler: this.applyTimeSettings.createDelegate(this)
                                }
                            ]
                        }
                    ] // timeset items
                }
            );
            this.timeForm = new Ext.FormPanel(
                {
                    padding: 6,
                    ref: 'timeForm',
                    id: 'timeform',
                    scope: this,
                    items: [
                        this.timeSet
                    ]
                }
            );


            this.restart = new Ext.form.FieldSet(
                {
                    title:  _("Restart digitalSTROM server application"),
                    scope:  this,
                    items: [
                        {
                            xtype:  'panel',
                            padding: 3,
                            hideBorders: true,
                            ref: 'dssUptime',
                            bodyBorder: false,
                            cls: 'x-green-text',
                            scope: this,
                            autoLoad: { 
                                scope: this,
                                url: '/cgi-bin/dssuptime.hs'
                            }
                        },
                        {
                            xtype: 'button',
                            text: _("Restart dSS"),
                            minWidth: 80,
                            handler: this.restartdSS.createDelegate(this)
                        }
                    ]
                }
            );

            this.reboot = new Ext.form.FieldSet(
                {
                    title:  _("Reboot system"),
                    scope:  this,
                    items: [
                        {
                            xtype:  'panel',
                            padding: 3,
                            bodyBorder: false,
                            hideBorders: true,
                            ref: 'Uptime',
                            cls: 'x-green-text',
                            autoLoad: '/cgi-bin/dss11uptime.hs'
                        },
                        {
                            xtype: 'button',
                            text: _("Reboot"),
                            minWidth: 80,
                            handler: this.reboot.createDelegate(this)
                        }
                    ]
                }
            );

            this.zoneStore.load(); 

            Ext.apply(this, {
                hideBorders: true,
                items: [
                    this.nameForm,
                    this.timeForm,
                    {
                        layout: 'column',
                        bodyBorder: false,
                        padding: 3,
                        items: [
                            {
                                columnWidth: .5,
                                padding: 3,
                                xtype: 'panel',
                                bodyBorder: false,
                                items: [ this.restart ]
                            },
                            {
                                columnWidth: .5,
                                padding: 3,
                                xtype: 'panel',
                                bodyBorder: false,
                                items: [ this.reboot ]
                            }
                        ]
                    }
                ]
            });
            
            dSS11.SysConfigPanel.superclass.initComponent.apply(this, arguments);

           this.on('activate',
                    function() {
                        this.reloadData.defer(1000, this);
                    },
                    this);

        }, // initcomponent
        restartdSS: function() {
            Ext.Msg.show({
                title: _("Restart dSS application"),
                msg: _("Are you sure that you want to restart the dSS application?"),
                    buttons: Ext.Msg.YESNO,
                    scope: this,
                    fn: function(button, text) {
                        if (button == 'yes') {
                            this.el.mask(_("Restarting dSS application") + 
                                            "...", 'x-mask-loading');

                            Ext.Ajax.request({
                                url: '/cgi-bin/restartdss.sh',
                                disableCaching: true,
                                //params: { 'dss' : 'dss' }, // dummy parameter
                                method: 'GET',
                                scope: this,
                                success: function(response) {
                                    this.el.unmask();
                                    try {
                                        var jsonData = Ext.util.JSON.decode(
                                                response.responseText);

                                        if (jsonData.success !== true) {
                                            Ext.Msg.alert(_("Error"), 
                                                            jsonData.message);
                                            return;
                                        } else {
                                            Ext.Msg.show({
                                                title: _("Info"),
                                                msg: _("digitalSTROM " + 
                                                "server application " + 
                                                "successfully restarted"),
                                                icon: Ext.MessageBox.INFO,
                                                buttons: Ext.Msg.OK
                                            });
                                        }
                                    } catch (error) {
                                        Ext.Msg.alert(_("Error"),
                                            _("digitalSTROM server " + 
                                              "application could not be " + 
                                              "restarted") + ": " + 
                                                error.message);
                                    }
                                    this.restart.dssUptime.doAutoLoad();
                                },
                                failure: function() {
                                    this.el.unmask();
                                    Ext.Msg.alert(_("Error"),
                                        _("digitalSTROM server " + 
                                          "application could not be " + 
                                          "restarted"));
                                    this.restart.dssUptime.doAutoLoad();
                                }
                            });

                        }
                    },
                    animEl: 'elId',
                    icon: Ext.MessageBox.QUESTION
            });

        },
        rebootWaitMsg: function(secs, w) {
            if (secs > 0) {
                w.updateProgress(1-(secs/dSS11.config.rebootWait), secs + " " +
                        dSS11.config.gt.ngettext("second remaining", 
                                                 "seconds remaining",
                                                  secs));
                secs--;
                this.rebootWaitMsg.defer(1000, this, [secs, w]);
            } else {
                w.hide();
                this.findParentByType(Ext.Viewport).el.mask(_("Reloading page") + "...");
                var params = Ext.urlDecode(window.location.search.substring(1));
                params.select = 3;
                window.location = window.location.protocol + '//' +
                                  window.location.host + '/?' + 
                                  Ext.urlEncode(params);

            }
        },
        reboot: function() {
            Ext.Msg.show({
                title: _("Reboot system"),
                msg: _("Are you sure that you want to reboot the system?") +
                       "<br/>" +
                       _("Note: this will take about a minute"),
                    buttons: Ext.Msg.YESNO,
                    scope: this,
                    fn: function(button, text) {
                        if (button == 'yes') {
                            Ext.Ajax.request({
                                url: '/cgi-bin/rebootwrapper.sh',
                                disableCaching: true,
                                params: { action : 'reboot' },
                                method: 'GET',
                                scope: this,
                                success: function(response) {
                                    this.el.unmask();
                                    try {
                                        var jsonData = Ext.util.JSON.decode(
                                                response.responseText);

                                        if (jsonData.success !== true) {
                                            Ext.Msg.alert(_("Error"), 
                                                            jsonData.message);
                                            return;
                                        } else {
                                            var w = Ext.Msg.progress(
                                                _("Rebooting system"),
                                                _("Please wait while the " +
                                                  "system is rebooting") + 
                                                  "<br/>" + 
                                                _("The setup page will be " +
                                                  "automatically reloaded " +
                                                  "after the time elapses"));
                                            this.rebootWaitMsg(
                                                    dSS11.config.rebootWait, w);
                                        }
                                    } catch (error) {
                                        Ext.Msg.alert(_("Error"),
                                            _("System reboot could not " + 
                                              "be initiated") + ": " + 
                                                error.message);
                                    }
                                },
                                failure: function() {
                                    this.el.unmask();
                                    Ext.Msg.alert(_("Error"),
                                        _("System reboot could not be " + 
                                          "initiated"));
                                }
                            });

                        }
                    },
                    animEl: 'elId',
                    icon: Ext.MessageBox.QUESTION
            });
        },
        setSystemName:  function() {
            if (!this.nameForm.getForm().isValid()) {
                Ext.Msg.alert(_("Error"), _("The name you entered is invalid! Please do not use any whitespaces or special characters."));
                return;
            }

            this.el.mask(_("Applying changes") + '...', 'x-mask-loading');

            this.nameForm.getForm().submit({ 
                url: '/cgi-bin/setnamewrapper.sh',
                method: 'GET',
                scope: this,
                success: function(form, action) {
                    this.el.unmask();
                },
                failure: function(form, action) {
                    this.el.unmask();
                    switch (action.failureType) {
                        case Ext.form.Action.CONNECT_FAILURE:
                            Ext.Msg.alert(_("Error"),
                                _("Ajax communication failed"));
                        break;
                        default:
                            Ext.Msg.alert(_("Error"),
                                    _("Could not load installation name"));
                        break;
                    }
                }
            });
        },
        validateHostName: function(name) {
            if ((name.length === 0) || (name.length > 64)) {
                return false;
            }

            var expr = /[^0-9a-zA-Z\-]/;
            if (expr.exec(name) === null) {
                return true;
            }

            return false;
        },
        reloadHostName: function() {
            this.nameForm.el.mask(_("Loading system name") + "...",
                      'x-mask-loading');
                        
            this.nameForm.load({
                url: '/cgi-bin/getname.sh',
                method: 'GET',
                scope: this,
                disableCaching: true,
                success: function(form, action) {
                    try {
                        var jsonData = Ext.util.JSON.decode(
                                                action.response.responseText);

                        if (jsonData.success !== true) {
                            Ext.Msg.alert(_("Error"), jsonData.message);
                            this.nameForm.el.unmask();
                            return;
                        }
                    } catch (error) {
                        Ext.Msg.alert(_("Error"),
                            _("Could not load installation name") +
                                ": "  + error.message);
                    }

                    this.nameForm.el.unmask();
                },
                failure: function(form, action) {
                    switch (action.failureType) {
                        case Ext.form.Action.CONNECT_FAILURE:
                            Ext.Msg.alert(_("Error"),
                                _("Ajax communication failed"));
                        break;
                        default:
                            Ext.Msg.alert(_("Error"),
                                    _("Could not load installation name"));
                        break;
                    }
                        
                    this.nameForm.el.unmask();
                }
            });
        },
        reloadTimeSettings: function() {
            this.timeForm.el.mask(_("Loading date and time settings") + "...", 
                      'x-mask-loading');
        
            this.timeForm.load({
                url: '/cgi-bin/dateinfo.sh',
                method: 'GET',
                scope: this,
                disableCaching: true,
                waitMsgTarget: this.timeForm.el,
                success: function(form, action) {
                    try {
                        var jsonData = Ext.util.JSON.decode(
                                                action.response.responseText);

                        if (jsonData.success !== true) {
                            Ext.Msg.alert(_("Error"), jsonData.message);
                            this.timeForm.el.unmask();
                            return;
                        }

                        dSS11.config.dSS11TimeDiff = dSS11.config.dSS11TimeDiff = (Date.now() - (jsonData.data.utcsecs * 1000));
                        if (dSS11.config.timeUpdateTimeout !== 0) {
                            clearTimeout(dSS11.config.timeUpdateTimeout);
                        }
                        dSS11.config.timeZoneShort = jsonData.data.zoneshort;
                        dSS11.time.update();
                    } catch (error) {
                        Ext.Msg.alert(_("Error"),
                            _("Could not load date and time settings") +
                                ": "  + error.message);
                    }
                            
                    this.timeForm.el.unmask();
                },
                failure: function(form, action) {
                    switch (action.failureType) {
                        case Ext.form.Action.CONNECT_FAILURE:
                            Ext.Msg.alert(_("Error"),
                                _("Ajax communication failed"));
                        break;
                        default:
                            Ext.Msg.alert(_("Error"),
                                    _("Could not load date and time settings"));
                        break;
                    }
                    this.timeForm.el.unmask();
                }
            });
        },
        applyTimeSettings:  function() {
            if (!this.timeForm.getForm().isValid()) {
                Ext.Msg.alert(_("Error"), _("The data you entered is invalid!"));
                return;
            }

            this.el.mask(_("Applying changes") + '...', 'x-mask-loading');

            this.timeForm.getForm().submit({ 
                url: '/cgi-bin/setdatewrapper.sh',
                method: 'GET',
                scope: this,
                success: function(form, action) {
                    this.el.unmask();
                    this.timePanel.ntpPanelSwitch.datePanel.date.setValue("");
                    this.timePanel.ntpPanelSwitch.datePanel.time.clearValue();
                    this.reloadTimeSettings();
                },
                failure: function(form, action) {
                    this.el.unmask();
                    switch (action.failureType) {
                        case Ext.form.Action.CONNECT_FAILURE:
                            Ext.Msg.alert(_("Error"),
                                _("Ajax communication failed"));
                        break;
                        default:
                            Ext.Msg.alert(_("Error"),
                                    _("Could not set date and time") +
                                    ": " + _(action.result.message));
                        break;
                    }
                }
            });
        },

        reloadData: function() {
            this.reloadHostName();
            this.reloadTimeSettings();
        },
        NTPChecked: function(box, checked) {
            if (checked === true) {
                this.timePanel.ntpPanelSwitch.datePanel.setVisible(false);
                this.timePanel.ntpPanelSwitch.ntpPanel.setVisible(true);
            } else {
                this.timePanel.ntpPanelSwitch.ntpPanel.setVisible(false);
                this.timePanel.ntpPanelSwitch.datePanel.setVisible(true);
            }

        }
    }
);

Ext.reg('dss11sysconfig', dSS11.SysConfigPanel);

