Ext.namespace('dSS11', 'dSS11.config');

dSS11.SystemUpdatePanel = Ext.extend(Ext.Panel,
    {
        initComponent: function()
        {
            this.updateStore = new Ext.data.JsonStore(
                {
                    url: '/cgi-bin/opkgwrapper.sh?action=list-upgradable',
                    root: 'data',
                    fields: 
                    [
                        'name', 'version', 'maintainer', 
                        'description', 'homepage', 'state'
                    ]
                }
            );

            this.loaded = false;
            this.fromRefetch = false;

            var renderPackage = function(value, p, record)
            {
                var hp = record.get('homepage');
                if (hp !== "") {
                    hp = String.format('&nbsp;&nbsp;<a href="{0}" target="_blank">{0}</a>',
                            hp);
                }

                return String.format('<div class="{0}">' + 
                        '<span class="{4}"><b>{1}</b></span>{2}<br/>' + 
                        '<span class="{4}">' + _("Version")+
                        ':&nbsp;{3}</span></div>' +
                        '<br/><div class="{4}">{5}' +
                        '</div>',
                        'x-green-text x-text-link',                 // {0}
                        record.get('name'),                         // {1}
                        hp,                                         // {2}
                        record.get('version'),                      // {3}
                        'black-text',                               // {4}
                        record.get('description')                   // {5}
                );
            };

            this.mainGrid = new Ext.grid.GridPanel(
                {
                    store: this.updateStore,
                    columns:
                    [
                        {
                            id: 'addon',
                            header: _("Available updates"),
                            dataIndex: 'name',
                            sortable: true,
                            renderer: renderPackage
                        }
                    ],
                    trackMouseOver: false,
                    viewConfig: { forceFit: true },
                    disableSelection: true,
                    bodyBorder: false
                }
            );

            this.noUpdatesTooltip =  new Ext.ToolTip(
                {
                    target: 'updateButton',
                    html: '<br/>' + 
                          _("Your system is up to date"),
                    title: _("No updates found"),
                    closable: true,
                    draggable: true,
                    anchor: 'right'
                }
            );

            this.gotUpdatesTooltip = new Ext.ToolTip(
                {
                    target: 'updateButton',
                    html: '<br/>' + 
                          _("Updates are available for your system, click the update button to install them"),
                    title: _("Updates found"),
                    closable: true,
                    draggable: true,
                    anchor: 'right'
                }
            );

            Ext.apply(this,
                {
                    layout: 'fit',
                    tbar: [
                        {
                            xtype: 'button',
                            text: _("Refresh view"),
                            iconCls: 'icon-refresh-view',
                            scope: this,
                            handler: this.refreshList.createDelegate(this)
                        },
                        {
                            xtype: 'button',
                            text: _("Check for new updates"),
                            iconCls: 'icon-refresh-feed',
                            scope: this,
                            tooltip: _("Contact the server and check availability of new updates"),
                            handler: this.handleRefetch.createDelegate(this)
                        },
                        '->',
                        {
                            xtype: 'button',
                            text: _("Update system"),
                            ref: 'updateButton',
                            id: 'updateButton',
                            iconCls: 'icon-package-update',
                            disabled: true,
                            scope: this,
                            handler: this.updateSystem.createDelegate(this)
                        }
                    ],
                    items: [ this.mainGrid ]
                }
            );

            dSS11.SystemUpdatePanel.superclass.initComponent.apply(this, arguments);

            this.on('activate',
                function() 
                {
                    if (this.loaded === false) {
                        this.updateStore.load();
                        this.loaded = true;
                    }
                },
                this);

            this.on('hide',
                function()
                {
                    this.noUpdatesTooltip.hide();
                    this.gotUpdatesTooltip.hide();
                },
                this);

            this.updateStore.on('load',
                function() 
                {
                    var button = this.getTopToolbar().get('updateButton');
                    var count = this.updateStore.getTotalCount();

                    if (count > 0) {
                        this.noUpdatesTooltip.hide();
                        this.noUpdatesTooltip.disable();
                        this.gotUpdatesTooltip.enable();
                        button.enable();
                        this.gotUpdatesTooltip.setTitle(
                                count + " " + 
                                dSS11.config.gt.ngettext("Update", 
                                                    "n% Updates", count) +
                                        " " + _("found"));

                        this.gotUpdatesTooltip.show();
                    } else {
                        button.disable();
                        this.gotUpdatesTooltip.hide();
                        this.gotUpdatesTooltip.disable();
                        this.noUpdatesTooltip.enable();
                        if (this.fromRefetch === true) {
                            this.noUpdatesTooltip.show();
                            this.fromRefetch = false;
                        }
                    }
                },
                this);

        },
        refreshList: function() 
        {
            this.updateStore.load();
        },
        updateDone: function()
        {
            var msg = _("Your system was successfully updated");
            Ext.Msg.show({
                title: _("Info"),
                msg: msg,
                icon: Ext.MessageBox.INFO,
                buttons: Ext.Msg.OK
            });
            this.refreshList();
        },
        refetchDone: function() 
        {
            this.fromRefetch = true;
            this.refreshList();
        },
        handleRefetch: function()
        {
            var fn = dSS11.util.opkg.createDelegate(this,
                           [ 'update',
                              this.findParentByType('viewport').el,
                              this.refetchDone.createDelegate(this) ]);
            fn();
        },
        updateSystem: function() {
            var fn = dSS11.util.opkg.createDelegate(this,
                           [ 'upgrade',
                              this.findParentByType('viewport').el,
                              this.updateDone.createDelegate(this) ]);
            fn();
        }
    }
);

Ext.reg('dss11systemupdate', dSS11.SystemUpdatePanel);

