Ext.namespace("dSS11", "dSS11.util");

// updateType: system/addon
dSS11.util.checkUpdateProgress = function(updateType, maskElement, doneCallback)
{
    var failMsg = _("Could not get current status");

    if (updateType === 'system') {
        failMsg = _("Could not get update status") + " " +
                  _("It is possible that the system " +
                    "webserver is currently being updated. " +
                    "Please try reloading this page in a " +
                    "few minutes.");
    }

    Ext.Ajax.request({
        url: '/cgi-bin/updatestatuswrapper.sh',
        disableCaching: true,
        method: 'GET',
        scope: this,
        params: { type: updateType },
        success: function(response) {
            try {
                var jsonData = Ext.util.JSON.decode( response.responseText);

                if (jsonData.success !== true) {
                    Ext.Msg.alert(_("Error"), _(jsonData.message));
                    maskElement.unmask();
                    return;
                } else {
                    if(jsonData.code == dSS11.config.codes.ERR_DOWAIT){
                        var task = new Ext.util.DelayedTask(
                            dSS11.util.checkUpdateProgress, this, 
                            [ updateType, maskElement, doneCallback ]);
                        task.delay(5000);
                    } else if (jsonData.code ==
                                           dSS11.config.codes.ERR_OK) {
                        maskElement.unmask();
                        if (doneCallback !== undefined) {
                            doneCallback();
                        }
                    } else {
                        maskElement.unmask();
                        Ext.Msg.alert(_("Error"), _(jsonData.message));
                    }

                }
            } catch (error) {
                maskElement.unmask();
                Ext.Msg.alert(_("Error"),
                    _("Cold not determine current status") + ": " +
                        _(error.message));
            }
        },
        failure: function() {
             maskElement.unmask();
             Ext.Msg.alert(_("Error"), failMsg);
        }
    });
};

dSS11.util.opkg = function(action, maskElement, doneCallback, name) 
{
    var message = _("Please wait"); 

    if ((action === 'update') || (action === 'addon-update')) {
        message = _("Downloading feed lists from server");
    } else if ((action === 'upgrade') || (action === 'addon-upgrade')) {
        message = _("Performing update, this may take several minutes");
    } else if (action === 'addon-install') {
        message = _("Please wait, installing add-on") + " <b>" + name + "</b>";
    } else if (action === 'addon-remove') {
        message = _("Please wait, removing add-on") + " <b>" + name + "</b>";
    }
    maskElement.mask(message + "...",
                     'x-mask-loading');

    Ext.Ajax.request({
        url: '/cgi-bin/opkgwrapper.sh',
        disableCaching: true,
        method: 'GET',
        scope: this,
        params: { action: action, name: name },
        success: function(response)
        {
            try {
                    var jsonData = Ext.util.JSON.decode(
                                                response.responseText);
                    if (jsonData.success !== true) {
                        Ext.Msg.alert(_("Error"), _(jsonData.message));
                        maskElement.unmask();
                        return;
                    } else {
                        var type = 'system';
                        if (action.slice(0, 5) === 'addon') {
                            type = 'addon';
                        }
                        var fn = dSS11.util.checkUpdateProgress.createDelegate(
                                            this, 
                                            [ 
                                                type, 
                                                maskElement, 
                                                doneCallback 
                                            ]);
                        fn();
                    }
            } catch (error) {
                maskElement.unmask();
                Ext.Msg.alert(_("Error"),
                    _("Failed to download feed lists from the server") +
                    ": " + _(error.message));
            }
        },
        failure: function() {
             maskElement.unmask();
             Ext.Msg.alert(_("Error"), _("Could not request new feed" +
                           " lists"));
        }
    });
};

